/*
 * Copyright 2002 3plus4 software.
 * All Rights Reserved.
 */
package de.iiipiv.game;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * @version 	1.0
 * @author	sma@3plus4.de
 */
public class JHexMap extends JComponent {
    public Image[] backgrounds = new Image[] { load("bg1"), load("bg2")};
    public Image[] objects = new Image[] { load("ob1")};
    public int hexheight = 53;
    public int hexheight2 = 26;
    public int hexwidth = 42;
    public int hexwidth2 = 17;

    private HexMapModel model;

    private Image selectionImage = load("sel");
    private Coord selectionCoord;

    public JHexMap() {
        this(new HexMapModel(256, 256));
    }

    public JHexMap(HexMapModel model) {
        this.model = model;
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                Coord c = fieldAt(e.getX(), e.getY());
                if (c != null) {
                    setSelection(c);
                    JHexMap.this.model.getField(c.x, c.y).background = 1;
                    if (Math.random() < 0.2)
                        JHexMap.this.model.getField(c.x, c.y).object ^= 1;
                }
            }
        });
    }

    /** Returns the current hex map model. */
    public HexMapModel getModel() {
        return model;
    }

    /** Paints this component. */
    protected void paintComponent(Graphics g) {
        if (model == null) {
            super.paintComponent(g);
            return;
        }
        Rectangle rc = g.getClipBounds();
        Rectangle rf = new Rectangle(2, 2, hexwidth + hexwidth2 + 4, hexheight + 6);
        for (int y = 0; y < model.h; y++, rf.y += hexheight) {
            rf.x = 2;
            for (int x = 0; x < model.w; x++, rf.x += hexwidth) {
                int o = (x & 1) * hexheight2;
                rf.y += o;
                if (rc.intersects(rf))
                    paintField(g, rf.x, rf.y, model.getField(x, y));
                rf.y -= o;
            }
        }
        paintSelection(g);
    }

    protected void paintField(Graphics g, int x, int y, HexMapModel.Field field) {
        if (field.background >= 0) {
            g.drawImage(backgrounds[field.background], x, y, this);
            if (field.object > 0)
                g.drawImage(objects[field.object - 1], x, y, this);
        }
    }

    protected void paintSelection(Graphics g) {
        if (selectionCoord != null) {
            int o = (selectionCoord.x & 1) * hexheight2;
            g.drawImage(
                selectionImage,
                selectionCoord.x * hexwidth - 2 + 2,
                selectionCoord.y * hexheight + o - 1 + 2,
                this);
        }
    }

    /** Returns the component's preferred size. */
    public Dimension getPreferredSize() {
        if (model == null)
            return super.getPreferredSize();
        return new Dimension(
            model.w * hexwidth + hexwidth2 + 4,
            model.h * hexheight + hexheight2 + 6);
    }

    public void setSelection(Coord c) {
        repaint(selectionCoord);
        selectionCoord = c;
        repaint(selectionCoord);
    }

    public Coord getSelection() {
        return selectionCoord;
    }

    /** Queues a repaint event for the given field. Adds some pixel to support selections. */
    private void repaint(Coord c) {
        if (c != null)
            repaint(
                c.x * hexwidth,
                c.y * hexheight + (c.x & 1) * hexheight2 + 1,
                hexwidth + hexwidth2 + 4,
                hexheight + 6);
    }

    /** Returns the Coord of the field at (px,py). */
    protected Coord fieldAt(int px, int py) {
        if (model == null)
            return null;
        px -= 2;
        py -= 2;
        int x = px / hexwidth;
        py -= (x & 1) * hexheight2;
        int y = py / hexheight;
        px %= hexwidth;
        if (px <= hexwidth2) {
            py %= hexheight;
            if (py <= hexheight2) {
                if ((hexwidth2 - px) * hexheight2 > py * hexwidth2) {
                    x--;
                    y -= (x & 1);
                }
            } else {
                if (px * hexheight2 < (py - hexheight2) * hexwidth2) {
                    y += (x & 1);
                    x--;
                }
            }
        }
        if (x >= 0 && x < model.w && y >= 0 && y < model.h)
            return new Coord(x, y);
        return null;
    }

    /** Helper method to load an image. */
    private Image load(String s) {
        return new ImageIcon(getClass().getResource(s + ".png")).getImage();
    }

    public static void main(String[] arguments) {
        JFrame f = new JFrame("3plus4 HexMap Demo");
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        JHexMap m = new JHexMap();
        f.getContentPane().add(new JScrollPane(m));
        f.pack();
        f.show();
    }
}